#include <iostream.h>
#include <string.h>
#include <stdarg.h>
#include <new.h>
#include <stdlib.h>

#include "dynstrlb.h"
#include "listlib1.h"
#include "menu.h"

struct symbol {
   String name;
   short type;
   short storage;
   short level;
};

ostream &operator << (ostream &stream, symbol &obj)
{
	stream << "Name    : " << obj.name << endl;
	stream << "Type    : " << obj.type << endl;
	stream << "Storage : " << obj.storage << endl;
	stream << "level   : " << obj.level << endl;
	return stream;
}

istream &operator >> (istream &stream, symbol& obj)
{
	cout << "Name    : ";
	stream >> obj.name;
	cout << "Type    : ";
	stream >> obj.type;
	cout << "Storage : ";
	stream >> obj.storage;
	cout << "level   : ";
	stream >> obj.level;
	return stream;
}

#ifdef __SC__
#pragma template_access public
#pragma template Node<symbol>
#pragma template List<symbol>
#pragma template operator <<(ostream&,List<symbol>&)
#endif

ostream &operator << (ostream &stream, Node<symbol> &obj)
{
	stream << obj.data;
	return stream;
}

short compare(symbol in1, symbol in2)
{
	if (!strcmp(in1.name,in2.name) && (in1.level == in2.level))
		return 0;
	else
		return 1;
}

const int HASHSZ = 512;
const int HASHMSK = 0x01ff;

class hashTbl {
protected:
	List<symbol> **Tbl;
	int numsyms;
	long hash(char *name);
public:
	hashTbl();
	~hashTbl();
	void hashput(symbol& identifier);
	symbol *hashget(String name,int level);
	void prtTbl();
};

long hashTbl::hash(char *name)
{
	int i=0, len=strlen(name);
	static int cnt=0;
	
	// scatter, scramble & squeeze	
	long scatter=0;
	// scatter - get a unique number
	for (i=0; i < len; i++)
	{
		scatter += name[i] + i + HASHSZ;
	}
	
	// scramble - "scramble" the bits
	long shiftr = scatter >> 3;       // shift right
	long shiftl = scatter << 3;       // shift left
	long scramble = shiftr ^ shiftl;  // XOR
		
	// squeeze - force the number into our range
	long squeeze = scramble & HASHMSK;
	
	return squeeze;
}

hashTbl::hashTbl() 
{
	Tbl = new List<symbol> *[HASHSZ];
	for (int i=0; i < HASHSZ; i++)
		Tbl[i] = 0;
	numsyms = 0;
}

hashTbl::~hashTbl()
{
	for (int i=0; i < HASHSZ; i++)
		delete Tbl[i];
	delete Tbl;
	numsyms = 0;
}

void hashTbl::hashput(symbol& identifier)
{
	long key = hash(identifier.name);
	if (!Tbl[key])
		Tbl[key] = new List<symbol>;
		
	Tbl[key]->insert(identifier);
	numsyms++;
}

symbol *hashTbl::hashget(String name,int level)
{
	symbol check;
	symbol *outsym;
	long key = hash(name);
	
	check.name = name;
	check.level = level;
	check.type = 0;
	check.storage = 0;
	
	if (Tbl[key])
	{
		outsym = Tbl[key]->get(check);
		if (!outsym)
		{
			cout << name << " NOT in symbol table!" << endl;
			return 0;
		}
	}
	else
	{
		cout << name << " NOT in symbol table!" << endl;
		return 0;
	}
	
	return outsym;
}

void hashTbl::prtTbl()
{
	if (!numsyms)
	{
		cout << "Table is empty!" << endl;
		return;
	}
	
	for (int i=0; i < HASHSZ; i++)
	{
		if (Tbl[i])
		{
			cout << "Bucket<" << i << ">:" << endl;
			cout << *(Tbl[i]);
		}
	}
}

void exhausted()
{
	cout << "Exhausted the free store." << endl;
	exit(1);
}

int main()
{
	hashTbl hTbl;
	symbol variable;
	symbol *vp=0;
	String vname;
	int done=0,choice=0,vlevel=0;
	menu todo(5, 1, 0,
			 "<<<<<<<<<<<<<<<<  hash  >>>>>>>>>>>>>>>>",
			 "         1) Add symbol.",
			 "         2) Get symbol.",
			 "         3) Print Table.",
			 "         4) Exit.");	
	while (!done)
	{
		cout << todo;
		choice = todo.menu_choice();
		
		switch (choice) {
			case 1:
				cin >> variable;
				hTbl.hashput(variable);
				break;
			case 2:
				cout << "var Name : ";
				cin >> vname;
				cout << "val level: ";
				cin >> vlevel;
				vp = hTbl.hashget(vname,vlevel);
				if (vp)
				{
					cout << *vp;
					delete vp;
				}
				break;
			case 3:
				hTbl.prtTbl();
				break;
			case 4:
				done = 1;
		}
	}
	
	return 0;  
}
