// strlib.cpp
// this file includes the functions for the static array class String
#include <iostream.h>
#include <iomanip.h>
#include <assert.h>
#include <string.h>
#include "strlib.h"

String::String() { _sz = 0; }

String::String(char *instr)
{
	if (instr)
	{
		_sz = strlen(instr);
		assert(_sz < MAX);
		strcpy(_storage,instr);
	}
	else
		_sz = 0;
}
		
String::~String() { _sz = 0; }
		
// overload the [] for bounds checking
char &String::operator [](int index)
{
	assert(index < _sz);
	assert(index >= 0);
	return _storage[index];
}
		
// overload conversion to char * for
// passing String class to a function
String::operator char *()
{	return _storage; }


ostream &operator << (ostream &stream, String &inobj)
{
	stream << inobj._storage;
	return stream;
}

void String::oldcopy(char *instr)
{
	int len=0,i=0;
	char *sptr = _storage;
	
	if (instr)
	{
		len = strlen(instr);
		assert(len < MAX);
		_sz = len;
		while ( (*sptr++ = *instr++) != '\0');
	}
}

String &String::operator = (String &inobj)
{
	int i=0;
	char *sptr = _storage;
	char *instr = inobj._storage;
	
	_sz = inobj._sz;
	while ( (*sptr++ = *instr++) != '\0');

	return *this;
}	

String &String::operator = (char *instr)
{
	int len=0,i=0;
	char *sptr = _storage;
	
	if (instr)
	{
		len = strlen(instr);
		assert(len < MAX);
		_sz = len;
		while ( (*sptr++ = *instr++) != '\0');
	}

	return *this;
}

/* ***********************************************************
     FUNCTION NAME:  pad
     PURPOSE: pad a string to a specified length.
     INPUT:   padchar - character to pad the string with.
              padlen - length for resultant string.
     OUTPUT:  an integer which represents the number of characters in the string.
     AUTHOR: MCD
    ************************************************************ */
void String::pad(char pad_char, int pad_len)
{
	int i=0;
	assert(pad_len < MAX);
	for (i=_sz; i < pad_len; i++)
		_storage[i] = pad_char;
	_storage[pad_len] = '\0';
}

/* ***********************************************************
     FUNCTION NAME:  slen
     PURPOSE:  determine the length of the input string.  A mimic of strlen.
     OUTPUT:  an integer which represents the number of characters in the string.
     AUTHOR: MCD
    ************************************************************ */
int String::slen()
{
	int i;
	
	for (i=0; *(_storage+i); ++i);
	return(i);
}

/* ***********************************************************
     FUNCTION NAME:  search
     PURPOSE:  the routine searches for a substring within a string and returns the index
				of the start of the substring in the string.
     INPUT:  word - character string to search for.
     OUTPUT:  an integer which is the index in the searchstr where the word starts.
     AUTHOR: MCD
    ************************************************************ */
int String::search(char *word)
{
	int idx,i,j;
	char *searchstr = _storage;
	
	for (i=0; searchstr[i]; i++)
	{
		if (searchstr[i] == word[j])
		{
			if (!j) idx = i;
			j++;
		}
		else
			j = 0;
		
		if (word[j] == '\0')
			break;
	}

	if (word[j] != '\0')
		return(-1);
	else
		return(idx);
}

/* ***********************************************************
     FUNCTION NAME:  reverse
     PURPOSE:  reverse the characters in the input string.
     INPUT:  reverse - the character string to store the reversed characters.
     OUTPUT:  none.
     AUTHOR: MCD
    ************************************************************ */
String String::reverse()
{
	int i=0, len=0;
	String temp;
	char *reverse = temp._storage;
	char *instr = _storage;
	
	len = _sz;	
	reverse[len] = '\0'; /* null terminate */	
	while (instr[i] != '\0')
		reverse[--len] = instr[i++];
	return temp;				
}

istream &operator >> (istream &stream, String &inobj)
{
	char temp[256];
	stream >> temp;
	assert(strlen(temp) <= MAX);
	strcpy(inobj._storage,temp);
	inobj._sz = strlen(temp);
	return stream;
}

void String::getline(istream &stream)
{
	char temp[256];
	stream.getline(temp,255);
	strcpy(_storage,temp);
	_sz = strlen(temp);
}

short String::operator == (String &inobj)
{
	if (!strcmp(inobj._storage,_storage))
		return 1;
	else
		return 0;
}


